﻿#==============================================================================
# §package      DE_EPAGES::Ebay::API::Constants
# §state        public
#------------------------------------------------------------------------------
# §description  Some constants for Ebay
#==============================================================================
package DE_EPAGES::Ebay::API::Constants;
use base Exporter;
use utf8;

use strict;
## no critic qw(ProhibitAutomaticExportationNonDAL)
our @EXPORT = qw( 
  EBAYSTORERUNAMEPART
  AUCTION_EDITABLE
  AUCTION_VERIFIED
  AUCTION_RUNNING
  AUCTION_ENDED
  AUCTION_SCHEDULED
  AUCTION_FAILED
  AUCTION_COMPLETED
  REMOTE_API_PARALLEL_REQUESTS
  REMOTE_GETSELLERLIST_ENTRIESPERPAGE
  REMOTE_GETSELLERLIST_OUTPUTSELECTOR
  REMOTE_GETSELLERLIST_XML_API_URL
  REMOTE_GETSELLERLIST_XML_SANDBOX_API_URL
  REMOTE_CATALOG_REST_API_URL
  REMOTE_CATALOG_REST_SANDBOX_API_URL
  REMOTE_TAXONOMY_REST_SANDBOX_API_URL
  REMOTE_TAXONOMY_REST_API_URL
  REMOTE_GETSELLERLIST_UA_TIMEOUT
  MAX_TRIES_FOR_RENEW_OAUTH_TOKEN
  MAX_TRIES_FOR_SEND_REQUEST_TOKEN_ERROR
  EBAY_DETAILS
  EBAY_LISTINGSTATUS
  EBAY_ATTRIBUTEID_RETURN_ACCEPTED
  EBAY_VALUEID_RETURN_ACCEPTED
  EBAY_ATTRIBUTEID_ITEM_CONDITION
  EBAY_VALUEID_ITEM_CONDITION_NEW
  EBAY_VALUEID_ITEM_CONDITION_USED
  EBAY_ERROR_SELLER_NO_STORE
  EBAY_ERROR_WRONG_PAYPAL_ADDRESS
  EBAY_DEFAULT_EAN_VALUE
  EBAY_DEFAULT_ISBN_VALUE
  EBAY_DEFAULT_UPC_VALUE
  EBAYAPI_CALLVERSIONS
  EBAY_CATEGORIES_DB_NAME
  EBAY_CATEGORIES_DB_FILENAME
  EBAY_CATEGORIES_TMP_DB_NAME
  EBAY_CATEGORIES_TMP_DB_FILENAME
  EBAY_OAUTH_SCOPE
  EBAY_CLIENTAPP_OAUTH_SCOPE
  EBAY_REST_ROUTES
  @CSV_WARNING_CODES_EBAYAUCTION
  @NO_MERCHANT_MAIL_ATTRIBUTES
  DURATION_GOOD_TILL_CANCELED
  UNITY_DEFAULT_TEMPLATE
  UNITY_TEMPLATE_PATH
  UNITY_DEFAULT_PICTURE_TYPE
  UNITY_DEFAULT_PIC_VARIATION_TYPE
);
use vars qw ( @CSV_WARNING_CODES_EBAYAUCTION @NO_MERCHANT_MAIL_ATTRIBUTES );
use constant EBAYSTORERUNAMEPART => 'epages5-Store-';

use constant AUCTION_EDITABLE  => 0;
use constant AUCTION_VERIFIED  => 1;
use constant AUCTION_RUNNING   => 2;
use constant AUCTION_COMPLETED => 4;
use constant AUCTION_SCHEDULED => 8;
use constant AUCTION_FAILED    => 16;
use constant AUCTION_ENDED     => 32;    # ended but not completed


# urls for API request GetSellerList
use constant REMOTE_GETSELLERLIST_XML_API_URL => 'https://api.ebay.com/ws/api.dll';
use constant REMOTE_GETSELLERLIST_XML_SANDBOX_API_URL => 'https://api.sandbox.ebay.com/ws/api.dll';

# sandbox and live system urls for accessing the catalog api
use constant REMOTE_CATALOG_REST_API_URL => 'https://api.ebay.com/commerce/catalog/v1_beta';
use constant REMOTE_CATALOG_REST_SANDBOX_API_URL => 'https://api.sandbox.ebay.com/commerce/catalog/v1_beta';

# sandbox and live system urls for accessing the taxonomy api
use constant REMOTE_TAXONOMY_REST_API_URL => 'https://api.ebay.com/commerce/taxonomy/v1';
use constant REMOTE_TAXONOMY_REST_SANDBOX_API_URL => 'https://api.sandbox.ebay.com/commerce/taxonomy/v1';

# attention! max entries per page is limited to 200 by ebay
# in some cases a timeout can occur, if set to 200 and to much entries are fetched in parallel
# the best setting is maybe 100-150
use constant REMOTE_GETSELLERLIST_ENTRIESPERPAGE => 125;


# set to 0 for disabling parallel requests for API calls
# https://go.developer.ebay.com/developers/ebay/support/certification says: "Don't use more than 18 simultaneous threads to make API calls."
use constant REMOTE_API_PARALLEL_REQUESTS => 18;
# timeout for parallel user agent
use constant REMOTE_GETSELLERLIST_UA_TIMEOUT => 300;

# max tries to get a new OAuhToken by the renew token afterwards the Auth is deleted
use constant MAX_TRIES_FOR_RENEW_OAUTH_TOKEN => 3;

# max tries to get a new OAuth token and resend after X-EBAY-API-IAF-TOKEN errors
use constant MAX_TRIES_FOR_SEND_REQUEST_TOKEN_ERROR => 2;


#========================================================================================
# §global       REMOTE_GETSELLERLIST_OUTPUTSELECTOR | list of details to fetch using
#               GetSellerList call | array.string
#========================================================================================
use constant REMOTE_GETSELLERLIST_OUTPUTSELECTOR => qw (
  HasMoreItems
  ItemArray.Item.ItemID
  ItemArray.Item.ListingDetails.EndTime
  ItemArray.Item.SellingStatus.BidCount
  ItemArray.Item.SellingStatus.CurrentPrice
  ItemArray.Item.SellingStatus.ListingStatus
  ItemArray.Item.SellingStatus.QuantitySold
  PageNumber
  PaginationResult
  ReturnedItemCountActual
);

#========================================================================================
# §global       EBAY_DETAILS | List of details to get using GetEbayDetails
#               | array.string
#========================================================================================
use constant EBAY_DETAILS => qw (
  CountryDetails
  CurrencyDetails
  DispatchTimeMaxDetails
  ItemSpecificDetails
  PaymentOptionDetails
  RegionDetails
  RegionOfOriginDetails
  ReturnPolicyDetails
  ShippingCarrierDetails
  ShippingLocationDetails
  ShippingPackageDetails
  ShippingServiceDetails
  SiteDetails
  TaxJurisdiction
  TimeZoneDetails
  UnitOfMeasurementDetails
  URLDetails
);

#========================================================================================
# §global       EBAY_LISTINGSTATUS | auction listing status name to code number
#               | ref.hash
#========================================================================================
use constant EBAY_LISTINGSTATUS => {
    'Active'    => AUCTION_RUNNING,
    'Ended'     => AUCTION_ENDED,
    'Completed' => AUCTION_COMPLETED,
};

use constant EBAY_ATTRIBUTEID_RETURN_ACCEPTED => 3803;
use constant EBAY_VALUEID_RETURN_ACCEPTED     => 32040;
use constant EBAY_ATTRIBUTEID_ITEM_CONDITION  => 10244;
use constant EBAY_VALUEID_ITEM_CONDITION_NEW  => 10425;
use constant EBAY_VALUEID_ITEM_CONDITION_USED => 10426;

use constant EBAY_ERROR_SELLER_NO_STORE => 13003;
use constant EBAY_ERROR_WRONG_PAYPAL_ADDRESS => 732;

# url encoded OAuth scope for fetching OAuth2 tokens and their renewal
use constant EBAY_OAUTH_SCOPE => 'https%3A%2F%2Fapi.ebay.com%2Foauth%2Fapi_scope%2Fsell.account%20'.
                                 'https%3A%2F%2Fapi.ebay.com%2Foauth%2Fapi_scope%2Fsell.inventory';

use constant EBAY_CLIENTAPP_OAUTH_SCOPE => 'https%3A%2F%2Fapi.ebay.com%2Foauth%2Fapi_scope';

use constant EBAY_REST_ROUTES => {
    'CATALOG_PRODUCT_SEARCH' => '/product_summary/search', #Catalog API product search
    'TAXONOMY_CATEGORY_TREE' => '/category_tree/', #Taxonomy category tree
};



#====================================================================================================
# for products without a UPC/EAN/ISBN we need to send eBay a plaform specific string
#====================================================================================================
# §see        | AD-2267                     | ebay: It's not possible to offer products without EAN
# §structure  | 'SiteId' => 'String value'  | siteid is the specific eBay platform
# §example    | '16' => 'Nicht relevant'    | 16 = Austria, see Database/XML/EbayEnvironment.xml
#====================================================================================================
use constant EBAY_DEFAULT_EAN_VALUE => {
    '0' => 'Does not apply',        # US
    '2' => 'Does not apply',        # Canada
    '3' => 'Does not apply',        # UK
    '15' => 'Does not apply',       # Australia
    '16' => 'Nicht zutreffend',     # Austria
    '23' => 'Non applicable',       # Belgium / French
    '71' => 'Non applicable',       # France
    '77' => 'Nicht zutreffend',     # Germany
    '100' => 'Does not apply',
    '101' => 'Non applicabile',     # Italy
    '104' => '該当しません',
    '123' => 'Niet van toepassing', # Belgium / Dutch
    '146' => 'Niet van toepassing', # Netherlands
    '186' => 'No aplicable',        # Spain
    '193' => 'Nicht zutreffend',    # Switzerland
    '196' => '不适用',               # Taiwan
    '201' => '不适用',
    '203' => 'Does not apply',
    '205' => 'Does not apply',      # Ireland
    '207' => 'Does not apply',
    '210' => 'Non applicabile',
    '211' => 'Does not apply',
    '212' => 'Nie dotyczy',
    '216' => 'Does not apply',
    '218' => 'Ej tillämpligt',      # Sweden
    '220' => 'Không áp dụng',
    '223' => '&#19981; &#36866; &#29992;',
};
use constant EBAY_DEFAULT_ISBN_VALUE => {
    '0' => 'Does not apply',        # US
    '2' => 'Does not apply',        # Canada
    '3' => 'Does not apply',        # UK
    '15' => 'Does not apply',       # Australia
    '16' => 'Nicht zutreffend',     # Austria
    '23' => 'Non applicable',       # Belgium / French
    '71' => 'Non applicable',       # France
    '77' => 'Nicht zutreffend',     # Germany
    '100' => 'Does not apply',
    '101' => 'Non applicabile',     # Italy
    '104' => '該当しません',
    '123' => 'Niet van toepassing', # Belgium / Dutch
    '146' => 'Niet van toepassing', # Netherlands
    '186' => 'No aplicable',        # Spain
    '193' => 'Nicht zutreffend',    # Switzerland
    '196' => '不适用',               # Taiwan
    '201' => '不适用',
    '203' => 'Does not apply',
    '205' => 'Does not apply',      # Ireland
    '207' => 'Does not apply',
    '210' => 'Non applicabile',
    '211' => 'Does not apply',
    '212' => 'Nie dotyczy',
    '216' => 'Does not apply',
    '218' => 'Ej tillämpligt',      # Sweden
    '220' => 'Không áp dụng',
    '223' => '&#19981; &#36866; &#29992;',
};
use constant EBAY_DEFAULT_UPC_VALUE => {
    '0' => 'Does not apply',        # US
    '2' => 'Does not apply',        # Canada
    '3' => 'Does not apply',        # UK
    '15' => 'Does not apply',       # Australia
    '16' => 'Nicht zutreffend',     # Austria
    '23' => 'Non applicable',       # Belgium / French
    '71' => 'Non applicable',       # France
    '77' => 'Nicht zutreffend',     # Germany
    '100' => 'Does not apply',
    '101' => 'Non applicabile',     # Italy
    '104' => '該当しません',
    '123' => 'Niet van toepassing', # Belgium / Dutch
    '146' => 'Niet van toepassing', # Netherlands
    '186' => 'No aplicable',        # Spain
    '193' => 'Nicht zutreffend',    # Switzerland
    '196' => '不适用',               # Taiwan
    '201' => '不适用',
    '203' => 'Does not apply',
    '205' => 'Does not apply',      # Ireland
    '207' => 'Does not apply',
    '210' => 'Non applicabile',
    '211' => 'Does not apply',
    '212' => 'Nie dotyczy',
    '216' => 'Does not apply',
    '218' => 'Ej tillämpligt',      # Sweden
    '220' => 'Không áp dụng',
    '223' => '&#19981; &#36866; &#29992;',
};

# define the ebay API version here for a call, if not the default api version should used
use constant EBAYAPI_CALLVERSIONS => {
    #'GetCategories'   => 707,
    #'GetCategory2CS'  => 707,
    #'GetAttributesCS' => 707,
};

use constant EBAY_CATEGORIES_DB_NAME => 'EbayCategories';
use constant EBAY_CATEGORIES_TMP_DB_NAME => 'EbayCategoriesTmp';
use constant EBAY_CATEGORIES_DB_FILENAME => $ENV{EPAGES_CONFIG} . '/' . EBAY_CATEGORIES_DB_NAME . '.db';
use constant EBAY_CATEGORIES_TMP_DB_FILENAME => $ENV{EPAGES_CONFIG} . '/' . EBAY_CATEGORIES_TMP_DB_NAME . '.db';

@CSV_WARNING_CODES_EBAYAUCTION = qw(
    NO_EBAYAUCTIONS
    EBAYAUCTION_EMPTY_PRODUCT
    EBAYAUCTION_INVALID_PRODUCT
    INVALID_SHOPEBAYENVIRONMENT
    EBAYAUCTION_INVALID_TEMPLATE
    NO_PARENT_AUCTION
    EBAYAUCTION_EXISTS_NOT_EDITABLE
    WRONG_ATTRIBUTE_VALUE_FORMAT
);

@NO_MERCHANT_MAIL_ATTRIBUTES = qw(
    ClosedOrClosedTempByProvider
    ClosedByMerchant
    IsInternalTestShop
    IsTrialShop
    IsClosed
);

# duration value for offers / auctions which should never end
use constant DURATION_GOOD_TILL_CANCELED => 2147483647;

# object and file path for the Unity template
use constant UNITY_DEFAULT_TEMPLATE => '/EbayTemplatesUnity/Simple';
use constant UNITY_TEMPLATE_PATH    => "$ENV{'EPAGES_CARTRIDGES'}/DE_EPAGES/Ebay/Templates/EbayUnity";

# default picture type for normal and variation products in Unity shops
use constant UNITY_DEFAULT_PICTURE_TYPE       => 'ImageLarge';
use constant UNITY_DEFAULT_PIC_VARIATION_TYPE => 'ZoomView';

1;
