#==============================================================================
# package      DE_EPAGES::ShopVote::UI::Shop
# base         DE_EPAGES::Presentation::UI::Object
# state        private
#==============================================================================
package DE_EPAGES::ShopVote::UI::Shop;
use base DE_EPAGES::Presentation::UI::Object;
use DE_EPAGES::AppStore::API::Object::ScriptTagAppClient;
use DE_EPAGES::AppStore::API::Shop qw ( GetUnityScriptTagsApp );
use DE_EPAGES::ShopVote::API::Constants qw ( SEAL_VALIGN SEAL_HALIGN SEAL_VOFFSET SEAL_HOFFSET);
use DE_EPAGES::ShopVote::API::Shop qw( GetShopVoteScriptTagUrl GetShopVoteSealScriptTagUrl );
use Scalar::Util qw(looks_like_number);

use strict;

sub UnitySaveShopVoteSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $Form = $Servlet->form;
    my $hFormValues = $Form->formValues;
    my $isValid = 1;

    # get variables from servlet
    my $ShopVoteHTMLCode = $hFormValues->{'ShopVoteHTMLCode'}->[0];
    my $ShopVoteScriptCode = $hFormValues->{'ShopVoteScriptCode'}->[0];
    my $ShopVoteID = $hFormValues->{'ShopVoteID'}->[0];
    my $IsShopVoteSealVisible = $hFormValues->{'IsShopVoteSealVisible'}->[0];
    my $ShopVoteSealVAlignment = $hFormValues->{'ShopVoteSealVAlignment'}->[0] // SEAL_VALIGN;
    my $ShopVoteSealHAlignment = $hFormValues->{'ShopVoteSealHAlignment'}->[0] // SEAL_HALIGN;
    my $ShopVoteSealVOffset = ($hFormValues->{'ShopVoteSealVOffset'}->[0] && looks_like_number($hFormValues->{'ShopVoteSealVOffset'}->[0]) && $hFormValues->{'ShopVoteSealVOffset'}->[0] >= SEAL_VOFFSET)
                              ? $hFormValues->{'ShopVoteSealVOffset'}->[0]
                              : SEAL_VOFFSET;
    my $ShopVoteSealHOffset = ($hFormValues->{'ShopVoteSealHOffset'}->[0] && looks_like_number($hFormValues->{'ShopVoteSealHOffset'}->[0]) && $hFormValues->{'ShopVoteSealHOffset'}->[0] >= SEAL_HOFFSET)
                              ? $hFormValues->{'ShopVoteSealHOffset'}->[0]
                              : SEAL_HOFFSET;

    # get app client to communicate with the epagesJ REST API
    my $UnityScriptTagsAppClient = GetUnityScriptTagsApp($Shop);

    # get ShopVote script tags
    my $ShopVoteScriptTagUrl = GetShopVoteScriptTagUrl($Shop);
    my $ShopVoteSealScriptTagUrl = GetShopVoteSealScriptTagUrl($Shop);

    # HTML code and JavaScript code contains content
    if (length $ShopVoteHTMLCode && length $ShopVoteScriptCode) {
        # HTML code is invalid
        if (
            !(
                $ShopVoteHTMLCode =~ /\bsrt-customer-data\b/s
                &&
                $ShopVoteHTMLCode =~ /\bsrt-customer-email\b/s
                &&
                $ShopVoteHTMLCode =~ /\bsrt-customer-reference\b/s
            )
        ) {
            $isValid = 0;
            $Form->addFormError({
                'Name' => 'ShopVoteHTMLCode',
                'Reason' => 'INVALID_SHOPVOTE_CODE',
                'ViewAction' => 'UnityMBO-ViewShopVote'
            });
        }

        # JavaScript code is invalid
        if (!($ShopVoteScriptCode =~ /var myToken = "[a-zA-Z0-9]{0,32}";/s)) {
            $isValid = 0;
            $Form->addFormError({
                'Name' => 'ShopVoteScriptCode',
                'Reason' => 'INVALID_SHOPVOTE_CODE',
                'ViewAction' => 'UnityMBO-ViewShopVote'
            });
        }

        if ($isValid == 1) {
            # add ShopVote script tag (if it doesn't exist already)
            $UnityScriptTagsAppClient->addScriptTag($ShopVoteScriptTagUrl);
        } else {
            # delete script-tag if ShopVoteHTMLCode invalid
            $UnityScriptTagsAppClient->deleteScriptTag($ShopVoteScriptTagUrl);
        }
    } else {
        $isValid = 0;
        # delete script-tag if ShopVoteHTMLCode is empty
        $UnityScriptTagsAppClient->deleteScriptTag($ShopVoteScriptTagUrl);

        # only one code field is filled
        if (length $ShopVoteHTMLCode || length $ShopVoteScriptCode) {
            $Form->addFormError({
                'Name' => 'ShopVoteCodeIncomplete',
                'Reason' => 'INVALID_SHOPVOTE_CODE',
                'ViewAction' => 'UnityMBO-ViewShopVote'
            });
        }
    }

    # ShopVote seal should be visible
    if ($IsShopVoteSealVisible == 1) {
        # ShopVote ID exists
        if (length $ShopVoteID) {
            # Set display position values for badge
            my $JSONContainer = {
                'VAlign' => $ShopVoteSealVAlignment,
                'HAlign' => $ShopVoteSealHAlignment,
                'VOffset' => $ShopVoteSealVOffset,
                'HOffset' => $ShopVoteSealHOffset,
            };
            $Shop->set({'ShopVoteSealPosition' => $JSONContainer});

            # add ShopVote seal script tag
            $UnityScriptTagsAppClient->addScriptTag($ShopVoteSealScriptTagUrl);
        } else {
            # reset ShopVote seal variables
            $hFormValues->{'ShopVoteID'}->[0] = undef;
            $hFormValues->{'IsShopVoteSealVisible'}->[0] = 0;

            # delete ShopVote seal script tag
            $UnityScriptTagsAppClient->deleteScriptTag($ShopVoteSealScriptTagUrl);
        }
    } else {
        # delete ShopVote seal script tag
        $UnityScriptTagsAppClient->deleteScriptTag($ShopVoteSealScriptTagUrl);
    }

    # executed collected form errors at the end
    if ($isValid == 0) {
        $Form->executeFormError()
    }

    $self->SUPER::Save($Servlet);
    return;
}

1;
